"use client"

import { Icons } from "@/components/icons"
import { AspectRatio } from "@/components/ui/aspect-ratio"
import { Button } from "@/components/ui/button"
import { Card } from "@/components/ui/card"
import {
    Dialog,
    DialogContent,
    DialogDescription,
    DialogFooter,
    DialogHeader,
    DialogTitle,
    DialogTrigger,
} from "@/components/ui/dialog"
import { Form, FormControl, FormField, FormItem, FormMessage } from "@/components/ui/form"
import { Input } from "@/components/ui/input"
import { Label } from "@/components/ui/label"
import { Textarea } from "@/components/ui/textarea"
import { toast } from "@/components/ui/use-toast"
import { UserAvatar } from "@/components/user-avatar"
import { zodResolver } from "@hookform/resolvers/zod"
import { useSession } from "next-auth/react"
import Image from "next/image"
import { useRef, useState } from "react"
import { useForm } from "react-hook-form"
import { z } from "zod"
import { useUpdateProfile } from "../hooks/use-update-profile"
import { IProfile, IUser } from "../types"

const FormSchema = z.object({
    name: z.string().min(1, "Name can't be blank.").max(50, "Name can't be more than 50 characters"),
    bio: z.string().max(160, "Bio can't be more than 160 characters").optional(),
    location: z.string().max(30, "Location can't be more than 30 characters").optional(),
    website: z.string().max(100, "Website can't be more than 100 characters").optional(),
})

export const EditProfileModal = ({ user }: { user: IUser }) => {
    const { isLoading, mutate } = useUpdateProfile()

    const form = useForm<z.infer<typeof FormSchema>>({
        resolver: zodResolver(FormSchema),
    })

    form.setValue('name', user.name)
    form.setValue('bio', user.bio || "")
    form.setValue('location', user.location || "")
    form.setValue('website', user.website || "")

    const [chosenImages, setChosenImages] = useState<Partial<IProfile>>({
        banner: { url: user?.banner || "", file: undefined },
        image: { url: user?.image || "", file: undefined },
    })

    const bannerInputRef = useRef<HTMLInputElement>(null)
    const imageInputRef = useRef<HTMLInputElement>(null)

    async function onSave(formData: z.infer<typeof FormSchema>) {
        if (!user) return null

        const profile: IProfile = {
            name: formData.name,
            bio: formData.bio,
            location: formData.location,
            website: formData.website,
            banner: { url: user?.banner || "", file: chosenImages.banner?.file },
            image: { url: user?.image || "", file: chosenImages.image?.file },
        }

        mutate({
            profile,
            userId: user.id,
        })

        toast({
            description: "Successfully updated profile.",
        })
    }

    const chooseImage = async (event: any, type: string) => {
        const file = event.target.files[0]
        if (!file) return

        if (type === "banner" && bannerInputRef.current)
            bannerInputRef.current.value = ""

        if (type === "avatar" && imageInputRef.current)
            imageInputRef.current.value = ""

        const reader = new FileReader()

        reader.onloadend = () => {
            setChosenImages({
                [type]: { url: reader.result as string, file },
            })
        }

        reader.readAsDataURL(file)
    }

    if (!user) return null

    return (
        <Dialog>
            <DialogTrigger asChild>
                <Button variant="outline">Edit Profile</Button>
            </DialogTrigger>
            <DialogContent className="sm:max-w-[600px]">
                <DialogHeader>
                    <DialogTitle>Edit profile</DialogTitle>
                    <DialogDescription>
                        Make changes to your profile here. Click save when you are done.
                    </DialogDescription>
                </DialogHeader>
                <Form {...form}>
                    <form onSubmit={form.handleSubmit(onSave)} className="grid gap-3 py-3">
                        <Card className="relative w-full overflow-hidden">
                            <AspectRatio ratio={889 / 500} className="bg-slate-600  dark:bg-slate-400">
                                {chosenImages.banner?.url &&
                                    <Image
                                        src={chosenImages.banner?.url}
                                        alt={"user banner image"}
                                        fill
                                        priority
                                        className="object-center"
                                    />
                                }
                            </AspectRatio>
                            <input
                                type="file"
                                accept="image/*"
                                ref={bannerInputRef}
                                onChange={(e) => chooseImage(e, "banner")}
                                disabled={isLoading}
                            />
                            <div className="absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 flex space-x-3">
                                <Button
                                    variant="outline"
                                    size="icon"
                                    className="bg-opacity-50 dark:bg-opacity-50"
                                    onClick={() => bannerInputRef.current?.click()}
                                    disabled={isLoading}
                                >
                                    <Icons.camera />
                                </Button>
                                {chosenImages.banner?.url && (
                                    <Button
                                        variant="outline"
                                        size="icon"
                                        className="bg-opacity-50 dark:bg-opacity-50"
                                        onClick={() => setChosenImages({ banner: { url: "", file: undefined } })}
                                        disabled={isLoading}
                                    >
                                        <Icons.close />
                                    </Button>
                                )}
                            </div>
                        </Card>

                        <div className="absolute w-24 h-24 -mt-12 overflow-hidden">
                            <UserAvatar
                                user={{ username: user.username, image: chosenImages.banner?.url || null }}
                                className="object-cover"
                            />
                            <input
                                type="file"
                                accept="image/*"
                                ref={imageInputRef}
                                onChange={(e) => chooseImage(e, "image")}
                                disabled={isLoading}
                            />
                            <div className="absolute top-1/2 left-1/2 transform -translate-x-1/2 -translate-y-1/2 flex space-x-3">
                                <Button
                                    variant="outline"
                                    size="icon"
                                    className="bg-opacity-50 dark:bg-opacity-50"
                                    onClick={() => imageInputRef?.current?.click()}
                                    disabled={isLoading}
                                >
                                    <Icons.camera />
                                </Button>
                            </div>
                        </div>

                        <div className="grid grid-cols-4 items-center gap-4">
                            <Label htmlFor="name" className="text-right">
                                Name
                            </Label>
                            <FormField
                                control={form.control}
                                name="name"
                                render={({ field }) => (
                                    <FormItem>
                                        <FormControl>
                                            <Input id="name" className="col-span-3" disabled={isLoading} {...field} />
                                        </FormControl>
                                        <FormMessage />
                                    </FormItem>
                                )}
                            />
                        </div>

                        <div className="grid grid-cols-4 items-center gap-4">
                            <Label htmlFor="bio" className="text-right">
                                Bio
                            </Label>
                            <FormField
                                control={form.control}
                                name="bio"
                                render={({ field }) => (
                                    <FormItem>
                                        <FormControl>
                                            <Textarea id="bio" className="col-span-3 resize-none" disabled={isLoading} {...field} />
                                        </FormControl>
                                        <FormMessage />
                                    </FormItem>
                                )}
                            />
                        </div>

                        <div className="grid grid-cols-4 items-center gap-4">
                            <Label htmlFor="location" className="text-right">
                                Location
                            </Label>
                            <FormField
                                control={form.control}
                                name="location"
                                render={({ field }) => (
                                    <FormItem>
                                        <FormControl>
                                            <Input id="location" className="col-span-3" disabled={isLoading} {...field} />
                                        </FormControl>
                                        <FormMessage />
                                    </FormItem>
                                )}
                            />
                        </div>

                        <div className="grid grid-cols-4 items-center gap-4">
                            <Label htmlFor="website" className="text-right">
                                Website
                            </Label>
                            <FormField
                                control={form.control}
                                name="website"
                                render={({ field }) => (
                                    <FormItem>
                                        <FormControl>
                                            <Input id="website" className="col-span-3" disabled={isLoading} {...field} />
                                        </FormControl>
                                        <FormMessage />
                                    </FormItem>
                                )}
                            />
                        </div>
                    </form>
                </Form>
                <DialogFooter>
                    <Button type="submit" disabled={isLoading}>
                        {isLoading && (
                            <Icons.spinner className="mr-2 h-4 w-4 animate-spin" />
                        )}
                        Save changes
                    </Button>
                </DialogFooter>
            </DialogContent>
        </Dialog>
    )
}